module Velcro
  # Parsing flag options passed in by the CLI
  module Flags
    def self.included(base)
      base.send(:attr_reader, :includes)
    end

    # rubocop:disable Metrics/AbcSize
    def establish_inclusions(flags, options)
      options = Hash[options.map{|k, v| [k.to_s, v] }]
      inclusions = Hash[*flags.map { |k| [k, nil] }.flatten].merge(options).dup

      if inclusions.values.compact.empty?
        # If no options were specified include all options
        inclusions.each{|k, _| inclusions[k] = true }
      elsif inclusions.values.include?(true)
        # If any options were specified as false, default the rest to true
        inclusions.each{|k, _| inclusions[k] = false if inclusions[k].nil? }
      else
        # If options were specified, but not as false, default the rest to false
        inclusions.each{|k, _| inclusions[k] = true if inclusions[k].nil? }
      end

      @includes = inclusions
    end
    # rubocop:enable Metrics/AbcSize

    def included?(flag)
      includes[flag.to_s]
    end
  end
end
