require 'singleton'
require 'yaml'

module Velcro
  # Configuration settings for Velcro projects
  class Config
    include Singleton

    @defaults = {
      dotfiles: "#{Velcro.home}/.dotfiles",
      repo_url: 'git@github.com:velcro/dotfiles'
    }

    def initialize
      setup
      reset
    end

    def apply(config)
      config.each_pair { |k, v| send("#{k}=", v) }
    rescue NoMethodError => e
      method = e.message.match(/`(.*)\=/)[1]
      raise NoMethodError, "No configuration setting for :#{method} in #{default_options.join(', ')}"
    end

    def reset
      apply(defaults)
      load_rc
    end

    private

    def load_rc
      return unless File.exist?(rc_path)
      apply(YAML.load_file(rc_path)['config'])
    end

    def setup
      defaults.each_pair{|k, _| self.class.send(:attr_accessor, k.to_sym) }
    end

    def defaults
      self.class.instance_variable_get(:"@defaults")
    end

    def default_options
      @default_options ||= defaults.keys.map{|k| ":#{k}" }
    end

    def rc_path
      File.join(dotfiles, '.velcrorc')
    end
  end
end
