module Velcro
  module Installer
    # Parent class for velcro installers
    class Base
      attr_reader :dryrun

      def initialize(options = {})
        @dryrun = options[:dryrun] || false
      end

      def self.installer(options = {})
        return Installer::Mac.new(options)   if Velcro.osx?
        return Installer::Linux.new(options) if Velcro.linux?

        fail UnkownOperatingSystem, 'Velcro cannot recognize your OS'
      end

      def install(target)
        package_manager = manager_name(target)
        if package_manager.nil?
          puts "Uknown install procedure for #{target}"
          return
        else
          puts "Installing #{target} via #{package_manager}..."
        end

        return if dryrun
        # Resolves to something like:
        #    self.brew_install("mysql")
        send "#{package_manager}_install", target
      end

      private

      def manager_name(target)
        return :velcro if respond_to?(target.to_sym, true)
        return :brew   if brew?(target)
        return :cask   if cask?(target)
      end

      def velcro_install(target)
        send(target.to_sym)
      end

      def rvm
        `gpg --keyserver hkp://keys.gnupg.net --recv-keys 409B6B1796C275462A1703113804BB82D39DC0E3`
        `curl -sSL https://get.rvm.io | bash -s stable`
        `. #{Velco.home}/.rvm/scripts/rvm`
        # `rvm install #{Velcro.config.ruby_version}`
      end

      def oh_my_zsh
        `curl -L http://install.ohmyz.sh | sh`
      end
    end
  end
end
