require 'fileutils'

module Velcro
  # Class to build out the filesystem on the host machine
  class Scaffold
    attr_reader :dryrun

    def initialize(options = {})
      @dryrun = options[:dryrun] || false
    end

    def install(config)
      create_directories config['dirs']
      generate_files config['files']
    end

    private

    def create_directories(dirs)
      return if (dirs ||= []).empty?

      puts "\nCreating directories:"
      dirs.each do |dir|
        path = "#{Velcro.home}/#{dir}"
        dryrun ? puts("mkdir -p #{path}") : FileUtils.mkdir_p(path)
      end
    end

    def generate_files(files)
      return if (files ||= []).empty?

      puts "\nTouching Files:"
      files.each do |file|
        path = "#{Velcro.home}/#{file}"
        dryrun ? puts("touch #{path}") : FileUtils.touch(path)
      end
    end
  end
end
