module Velcro
  module Setter
    # Core logic for configuring the host system settings
    class Base
      attr_reader :dryrun

      def initialize(options = {})
        @dryrun = options[:dryrun] || false
      end

      def self.setter(options = {})
        return Setter::Mac.new(options)   if Velcro.osx?
        return Setter::Linux.new(options) if Velcro.linux?

        fail UnkownOperatingSystem, 'Velcro cannot recognize your OS'
      end

      def apply(setting)
        method, args = setting, nil

        if setting.is_a?(Hash)
          method = setting.keys.first
          args = setting.values.first
        end

        unless respond_to?(method.to_sym, true)
          puts "Uknown procedure to apply setting: #{setting}"
          return
        end

        send(method, args)
      end

      private

      def apply_with_velcro(target)
        send(target.to_sym)
      end

      def ssh(*_args)
        if File.exist?("#{Velcro.home}/.ssh/id_rsa.pub")
          puts 'id_rsa.pub exists. Skipping ssh keygen'
        else
          command = 'ssh-keygen'
          dryrun ? puts(command) : system(command)
        end
      end

      # rubocop:disable Metrics/AbcSize
      # rubocop:disable Metrics/CyclomaticComplexity
      # rubocop:disable Metrics/MethodLength
      # rubocop:disable Metrics/PerceivedComplexity
      def git(*args)
        options = args.first || {}
        if options['name']
          command = "git config --global user.name \"#{options['name']}\""
          dryrun ? puts(command) : system(command)
        end

        if options['email']
          command = "git config --global user.email \"#{options['email']}\""
          dryrun ? puts(command) : system(command)
        end

        servers = [options['server'], options['servers']].flatten.compact
        return unless servers.any?

        servers.each do |server|
          command = "cat ~/.ssh/id_rsa.pub | ssh #{server} \"mkdir -p ~/.ssh; cat >> ~/.ssh/authorized_keys\""
          dryrun ? puts(command) : system(command)
        end
      end
      # rubocop:enable Metrics/AbcSize
      # rubocop:enable Metrics/CyclomaticComplexity
      # rubocop:enable Metrics/MethodLength
      # rubocop:enable Metrics/PerceivedComplexity
    end
  end
end
