module Velcro
  # Symlink files from your dotfiles directory to $HOME
  class Symlinker
    attr_reader :dryrun, :force

    def initialize(options = {})
      @dryrun = options[:dryrun]
      @force  = options[:force]
    end

    # In any subdirectory of your .dotfiles repo,
    # create a file with the .symlink extension
    # which will be stripped when copied to your
    # home diretory with a "." prefix
    #
    # ex: ~/.dotfiles/irb/irbrc.symlink -> ~/.irbrc
    def symlink!
      symlinks.each do |symlink|
        filename = File.basename(symlink, '.symlink')
        target   = File.join(Velcro.home, ".#{filename}")
        next if skip?(target)

        puts "Symlinking #{target}"
        unless dryrun
          FileUtils.rm_rf(target) if force
          `ln -s "#{symlink}" "#{target}"`
        end
      end
    end

    def symlinked?(symlink)
      filename = File.basename(symlink, '.symlink')
      target   = File.join(Velcro.home, ".#{filename}")
      File.exist?(target)
    end

    def skip?(target)
      return false unless File.exist?(target)

      if force
        puts "#{target} exists. Forcing removal"
        return false
      end

      puts "#{target} exists. Skipping"
      true
    end

    def symlinks
      @symlinks ||= Dir.glob File.join(Velcro.config.dotfiles, '/*/**{.symlink}')
    end
  end
end
